; ==============================================
;          =  MOB Collision Test v1.0 =
;       Intellivision ROM by A.Chevallier
;                  August 2002
; ----------------------------------------------
;         Mail : a_chevallier@yahoo.com
;         Web  : http://knox.ac.free.fr
; ==============================================


; ----------------------------------------------
; ROM type
; ----------------------------------------------

		ORG		$5000
		ROMW		10


; ----------------------------------------------
; System addresses
; ----------------------------------------------

MOBX		EQU		$0000				; MOB registers
MOBY		EQU		$0008
MOBA		EQU		$0010
MOBC		EQU		$0018

VBLK0		EQU		$0100				; VBlank interrupt vector
VBLK1		EQU		$0101

BT		EQU		$0200				; BackTab

GRAM		EQU		$3800				; GRAM


; ----------------------------------------------
; EXEC routines
; ----------------------------------------------

ZEROM		EQU		$1738				; clear memory area
INLTXT	EQU		$187B				; draw text from inline data


; ----------------------------------------------
; Colors
; ----------------------------------------------

C_BLK		EQU		$0000				; Black
C_BLU		EQU		$0001				; Blue
C_RED		EQU		$0002				; Red
C_TAN		EQU		$0003				; Tan
C_DGR		EQU		$0004				; Dark Green
C_GRN		EQU		$0005				; Green
C_YEL		EQU		$0006				; Yellow
C_WHT		EQU		$0007				; White
C_GRY		EQU		$0008				; Grey
C_CYN		EQU		$0009				; Cyan
C_ORG		EQU		$000A				; Orange
C_BRN		EQU		$000B				; Brown
C_PNK		EQU		$000C				; Pink
C_LBL		EQU		$000D				; Light Blue
C_YGR		EQU		$000E				; Yellow-Green
C_PUR		EQU		$000F				; Purple


; ----------------------------------------------
; Variables
; ----------------------------------------------

CUR_MOB	EQU		$015C
NXT_FRM	EQU		$015D
KEY		EQU		$015E
HIDE_ST	EQU		$015F

RMOBX		EQU		$0300
RMOBY		EQU		$0302
RMOBA		EQU		$0304
RMOBA1	EQU		$0305
RMOBC		EQU		$0306


; ----------------------------------------------
; ROM header
; ----------------------------------------------

		BIDECLE	NULL				; MOB definition
		BIDECLE	NULL				; process table
		BIDECLE	Main				; main entry point
		BIDECLE	NULL				; bg picture base
		BIDECLE	STIC_INIT			; STIC
		BIDECLE	Title				; date & title
		DECLE		$0080				; I don't know exactly what to put here ...

NULL:		BIDECLE	$0000

STIC_INIT:	DECLE		C_GRN, C_GRN		; initial color stack
		DECLE		C_GRN, C_GRN

		DECLE		C_GRN				; border color


; ==============================================
; Date & title
; ==============================================

Title:
		; date
		DECLE		$0066

		; title
		STRING	'MOB Collision Test'
		DECLE		0

; ----------------------------------------------
; Modify author and date on title screen
; ----------------------------------------------

		PSHR		R5

		MVII		#C_WHT,	R3
		MVII		#BT+$3D,	R4
		JSR		R5,		INLTXT
		STRING	'Arnauld Chevallier'
		DECLE		0

		MVII		#BT+$D0,	R4
		JSR		R5,		INLTXT
		STRING	'2002 - AC -'
		DECLE		0

		PULR		R7				; return to Exec


; ==============================================
; Main entry point
; ==============================================

Main:
		MVII		#BT,		R4		; clear screen
		MVII		#$00F0,	R0
		JSR		R5,		ZEROM

		CLRR		R0				; let's initialize some variables
		MVO		R0,		CUR_MOB
		MVO		R0,		NXT_FRM
		MVO		R0,		KEY
		MVO		R0,		HIDE_ST

		SDBD						; set our VBlank interrupt vector
		MVII		#VBlank,	R0
		MOVR		R0,		R1
		SWAP		R1
		MVO		R0,		VBLK0
		MVO		R1,		VBLK1

		EIS

		JSR		R5,		wait_vbl	; we call this routine twice, as we don't
		JSR		R5,		wait_vbl	; know the initial interrupt's state

		MVII		#GRAM,	R4		; now, we can initialize GRAM with our pictures
		SDBD
		MVII		#Pics,	R5
		MVII		#$0010,	R0

GRAM_copy:
		MVI@		R5,		R1		; row 1 ...
		MVO@		R1,		R4
		MVI@		R5,		R1
		MVO@		R1,		R4
		MVI@		R5,		R1
		MVO@		R1,		R4
		MVI@		R5,		R1
		MVO@		R1,		R4
		MVI@		R5,		R1
		MVO@		R1,		R4
		MVI@		R5,		R1
		MVO@		R1,		R4
		MVI@		R5,		R1
		MVO@		R1,		R4
		MVI@		R5,		R1		; ... row 8
		MVO@		R1,		R4
		DECR		R0
		BNEQ		GRAM_copy

		MVII		#$0390,	R0		; initialize MOB's variables
		MVO		R0,		RMOBX
		MVII		#$0399,	R0
		MVO		R0,		RMOBX+1
		MVII		#$0090,	R0
		MVO		R0,		RMOBY
		MVO		R0,		RMOBY+1
		MVII		#$0800,	R0
		MVO		R0,		RMOBA
		MVO		R0,		RMOBA+1

; ----------------------------------------------
; Draw text for MOB status
; ----------------------------------------------

draw_text:	MVII		#C_TAN,	R3
		MVII		#BT+$15,	R4
		JSR		R5,		INLTXT
		STRING	'(1) XSize ='
		DECLE		0

		MVII		#BT+$29,	R4
		JSR		R5,		INLTXT
		STRING	'(2) XFlip ='
		DECLE		0

		MVII		#BT+$3D,	R4
		JSR		R5,		INLTXT
		STRING	'(3) YFlip ='
		DECLE		0

		MVII		#BT+$51,	R4
		JSR		R5,		INLTXT
		STRING	'(4) YSiz4 ='
		DECLE		0

		MVII		#BT+$65,	R4
		JSR		R5,		INLTXT
		STRING	'(5) YSiz2 ='
		DECLE		0

		MVII		#BT+$79,	R4
		JSR		R5,		INLTXT
		STRING	'(6) YRes  ='
		DECLE		0

		MVII		#BT+$A1,	R4
		JSR		R5,		INLTXT
		STRING	'(7) Change picture'
		DECLE		0

		MVII		#BT+$B5,	R4
		JSR		R5,		INLTXT
		STRING	'(8) Toggle MOB 0/1'
		DECLE		0

; ----------------------------------------------
; Main loop
; ----------------------------------------------

main_loop:
		JSR		R5,		wait_vbl	; let's synchronize

; MOB colors
; ----------------------------------------------

		MVII		#RMOBA,	R1		; draw active MOB in white
		ADD		CUR_MOB,	R1
		MVI@		R1,		R0
		ANDI		#$FFF8,	R0
		ADDI		#$0007,	R0
		MVO@		R0,		R1

		MVII		#RMOBA1,	R1		; draw inactive MOB in black
		SUB		CUR_MOB,	R1
		MVI@		R1,		R0
		ANDI		#$FFF8,	R0
		MVO@		R0,		R1

; show status of active MOB
; ----------------------------------------------

		MVI		HIDE_ST,	R0		; do we have to show it ?
		TSTR		R0
		BNEQ		show_col

		MVII		#$0086,	R2		; yellow '0'
		MVII		#$008E,	R3		; yellow '1'

		MVII		#RMOBX,	R1		; show XSize
		ADD		CUR_MOB,	R1

		MVO		R2,		BT+$21
		MVI@		R1,		R0
		ANDI		#$0400,	R0
		BEQ		xflip

		MVO		R3,		BT+$21

xflip:	MVII		#RMOBY,	R1		; show XFlip
		ADD		CUR_MOB,	R1

		MVO		R2,		BT+$35
		MVI@		R1,		R0
		ANDI		#$0400,	R0
		BEQ		yflip

		MVO		R3,		BT+$35

yflip:	MVO		R2,		BT+$49	; show YFlip
		MVI@		R1,		R0
		ANDI		#$0800,	R0
		BEQ		ysiz4

		MVO		R3,		BT+$49

ysiz4:	MVO		R2,		BT+$5D	; show YFlip
		MVI@		R1,		R0
		ANDI		#$0200,	R0
		BEQ		ysiz2

		MVO		R3,		BT+$5D

ysiz2:	MVO		R2,		BT+$71	; show YFlip
		MVI@		R1,		R0
		ANDI		#$0100,	R0
		BEQ		yres

		MVO		R3,		BT+$71

yres:		MVO		R2,		BT+$85	; show YRes
		MVI@		R1,		R0
		ANDI		#$0080,	R0
		BEQ		show_col

		MVO		R3,		BT+$85

; show collisions
; ----------------------------------------------

show_col:
		MVII		#C_BLK,	R3		; border collision test
		MVII		#RMOBC,	R1
		ADD		CUR_MOB,	R1
		MVI@		R1,		R0
		ANDI		#$0200,	R0
		BEQ		no_border_col

		MVII		#C_WHT,	R3

no_border_col:
		MVII		#BT+$DD,	R4
		JSR		R5,		INLTXT
		STRING	'(Border)'
		DECLE		0

		MVII		#C_BLK,	R3		; background collision test
		MVII		#RMOBC,	R1
		ADD		CUR_MOB,	R1
		MVI@		R1,		R0
		ANDI		#$0100,	R0
		BEQ		no_BG_col

		MVII		#C_WHT,	R3

no_BG_col:
		MVII		#BT+$E5,	R4
		JSR		R5,		INLTXT
		STRING	'(Bkg)'
		DECLE		0

		MVII		#C_BLK, 	R3		; MOB collision test
		MVII		#RMOBC,	R1
		ADD		CUR_MOB,	R1
		MVI@		R1,		R0
		ANDI		#$0003,	R0
		BEQ		no_MOB_col

		MVII		#C_WHT,	R3

no_MOB_col:
		MVII		#BT+$EA,	R4
		JSR		R5,		INLTXT
		STRING	'(Mob)'
		DECLE		0

; hand controller input
; ----------------------------------------------

		MVI		$01FF,	R3		; get left controller input
		CMPI		#$00FF,	R3		; something hit ?
		BNEQ		left_ok
		MVI		$01FE,	R3		; no : get right controller input
left_ok:	XORI		#$00FF,	R3
		CMPI		#$0020,	R3		; not a disc input ?
		BGE		not_disc
		TSTR		R3
		BEQ		not_disc

		MVII		#RMOBX,	R1
		ADD		CUR_MOB,	R1
		MVI@		R1,		R0

		MOVR		R0,		R4		; R4 = x
		ANDI		#$FF,		R4

		MOVR		R3,		R2		; go east ?
		ANDI		#$0002,	R2
		BEQ		check_west

		CMPI		#$A8,		R4		; x <= $A8 ?
		BGT		check_south

		INCR		R0				; x++

check_west:
		MOVR		R3,		R2		; go west ?
		ANDI		#$0008,	R2
		BEQ		check_south

		TSTR		R4				; x > 0 ?
		BEQ		check_south

		DECR		R0				; x--

check_south:
		MVO@		R0,		R1		; update RMOBX

		MVII		#RMOBY,	R1
		ADD		CUR_MOB,	R1
		MVI@		R1,		R0

		MOVR		R0,		R4		; R4 = y
		ANDI		#$7F,		R4

		MOVR		R3,		R2		; go south ?
		ANDI		#$0001,	R2
		BEQ		check_north

		CMPI		#$68,		R4		; y <= $68 ?
		BGT		main_loop

		INCR		R0				; y++

check_north:
		MOVR		R3,		R2		; go north ?
		ANDI		#$0004,	R2
		BEQ		moves_done

		TSTR		R4				; y > 0 ?
		BEQ		main_loop

		DECR		R0				; y--

moves_done:
		MVO@		R0,		R1		; update RMOBY
		B		main_loop

not_disc:
		MVI		KEY,		R0
		MVO		R3,		KEY		; save last key
		TSTR		R0				; previous key not released ?
		BNEQ		main_loop

		MVII		#RMOBX,	R1		; make R1 point to RMOBX
		ADD		CUR_MOB,	R1		; for the current MOB

		CMPI		#$0081,	R3		; key = '1' ?
		BNEQ		check_2

		MVI@		R1,		R0		; toggle XSize
		XORI		#$0400,	R0
		MVO@		R0,		R1

check_2:
		ADDI		#$0002,	R1		; now, make R1 point to RMOBY

		CMPI		#$0041,	R3		; key = '2' ?
		BNEQ		check_3

		MVI@		R1,		R0		; toggle XFlip
		XORI		#$0400,	R0
		MVO@		R0,		R1

check_3:
		CMPI		#$0021,	R3		; key = '3' ?
		BNEQ		check_4

		MVI@		R1,		R0		; toggle YFlip
		XORI		#$0800,	R0
		MVO@		R0,		R1

check_4:
		CMPI		#$0082,	R3		; key = '4' ?
		BNEQ		check_5

		MVI@		R1,		R0		; toggle YSiz4
		XORI		#$0200,	R0
		MVO@		R0,		R1

check_5:
		CMPI		#$0042,	R3		; key = '5' ?
		BNEQ		check_6

		MVI@		R1,		R0		; toggle YSiz2
		XORI		#$0100,	R0
		MVO@		R0,		R1

check_6:
		CMPI		#$0022,	R3		; key = '6' ?
		BNEQ		check_7

		MVI@		R1,		R0		; toggle YRes
		XORI		#$0080,	R0
		MVO@		R0,		R1

check_7:
		CMPI		#$0084,	R3		; key = '7' ?
		BNEQ		check_8

		ADDI		#$0002,	R1		; make R1 point to RMOBA
		MVI@		R1,		R0
		MOVR		R0,		R2
		ANDI		#$FF87,	R0
		ADDI		#$0010,	R2		; next picture
		ANDI		#$0078,	R2
		ADDR		R2,		R0
		MVO@		R0,		R1

check_8:
		CMPI		#$0044,	R3		; key = '8' ?
		BNEQ		check_e

		MVI		CUR_MOB,	R0		; toggle current MOB
		XORI		#$0001,	R0
		MVO		R0,		CUR_MOB

check_e:
		CMPI		#$0028, 	R3		; key = 'enter' ?
		BNEQ		main_loop

		MVI		HIDE_ST,	R0		; toggle show / hide status
		XORI		#$0001,	R0
		MVO		R0,		HIDE_ST
		TSTR		R0
		BEQ		draw_text

		MVII		#BT,		R4		; clear screen
		MVII		#$00DC,	R0
		JSR		R5,		ZEROM
		B		main_loop

; ----------------------------------------------
; Wait for next VBlank interrupt
; ----------------------------------------------

wait_vbl:
		PSHR		R5
wait_vbl0:
		MVI		NXT_FRM,	R0
		TSTR		R0
		BEQ		wait_vbl0

		CLRR		R0
		MVO		R0,		NXT_FRM

		PULR		R7

; ----------------------------------------------
; Our VBlank routine
; ----------------------------------------------

VBlank:
		PSHR		R5

		MVII		#RMOBX,	R4		; write MOBX
		MVII		#MOBX,	R5
		MVI@		R4,		R1
		MVO@		R1,		R5
		MVI@		R4,		R1
		MVO@		R1,		R5

		MVII		#MOBY,	R5		; write MOBY
		MVI@		R4,		R1
		MVO@		R1,		R5
		MVI@		R4,		R1
		MVO@		R1,		R5

		MVII		#MOBA,	R5		; write MOBA
		MVI@		R4,		R1
		MVO@		R1,		R5
		MVI@		R4,		R1
		MVO@		R1,		R5

		MVII		#MOBC,	R5		; read MOBC ...
		MVI@		R5,		R1
		MVO@		R1,		R4
		MVI@		R5,		R1
		MVO@		R1,		R4

		CLRR		R0				; ... and clear it
		MVO		R0,		MOBC
		MVO		R0,		MOBC+1

		MVII		#C_GRN,	R0		; green background
		MVO		R0,		$0028

		MVO		R0,		$0020		; enable display
		MVI		$0021,	R0		; color stack mode

		MVII		#$0001,	R0		; set the NXT_FRM flag
		MVO		R0,		NXT_FRM

		PULR		R7

; ----------------------------------------------
; Some tremendous graphixx :-)
; ----------------------------------------------

Pics:
		; a stupid smiley
		DECLE		$0018, $0024, $0042, $0081, $00A5, $00A5, $0081, $0081
		DECLE		$0081, $0081, $0085, $00B9, $0081, $0042, $0024, $0018

		; an aimer
		DECLE		$00C3, $0081, $0081, $0081, $0000, $0000, $0000, $0000
		DECLE		$0000, $0000, $0000, $0000, $0081, $0081, $0081, $00C3

		; a cross
		DECLE		$0018, $0018, $0018, $0018, $0018, $0018, $00FF, $00FF
		DECLE		$00FF, $00FF, $0018, $0018, $0018, $0018, $0018, $0018

		; just 4 little dots
		DECLE		$0081, $0000, $0000, $0000, $0000, $0000, $0000, $0000
		DECLE		$0000, $0000, $0000, $0000, $0000, $0000, $0000, $0081

		; just one BIG dot
		DECLE		$00FF, $00FF, $00FF, $00FF, $00FF, $00FF, $00FF, $00FF
		DECLE		$00FF, $00FF, $00FF, $00FF, $00FF, $00FF, $00FF, $00FF

		; a square
		DECLE		$00FF, $0081, $0081, $0081, $0081, $0081, $0081, $0081
		DECLE		$0081, $0081, $0081, $0081, $0081, $0081, $0081, $00FF

		; the running guy ... tired of running and having a break :-)
		DECLE		$0018, $0018, $0000, $003C, $007E, $005A, $005A, $0000
		DECLE		$0018, $0018, $0018, $0018, $0018, $0018, $0000, $0018

		; a robot (a familiar one ??)
		DECLE		$003C, $0042, $0042, $003C, $0018, $0018, $0024, $00DB
		DECLE		$007E, $0018, $007E, $0000, $007E, $0066, $00E7, $00C3
